# -*- coding: utf-8 -*-
import re
from pathlib import Path

import pytest

from commons_1c.platform_ import get_last_1c_exe_file_fullpath
from commons_1c.version import get_version_as_number, get_version_as_parts


def test_get_version_as_number():
    assert get_version_as_number("") == 0
    assert get_version_as_number("foo") == 0
    assert get_version_as_number("1") == 1000000000000
    assert get_version_as_number("1.1") == 1000100000000
    assert get_version_as_number("1.1.1") == 1000100010000
    assert get_version_as_number("1.1.1.1") == 1000100010001


def test_get_version_as_parts():
    assert get_version_as_parts("") == []
    assert get_version_as_parts("foo") == []
    assert get_version_as_parts("1") == ["1"]
    assert get_version_as_parts("1.1") == ["1", "1"]
    assert get_version_as_parts("1.1.1") == ["1", "1", "1"]
    assert get_version_as_parts("1.1.1.1") == ["1", "1", "1", "1"]


def test_get_last_1c_exe_file_path_1():
    file_path = get_last_1c_exe_file_fullpath()

    assert isinstance(file_path, Path)
    assert re.match(
        r"(?i)c:\\Program Files \(x86\)\\1cv8\\\d+\.\d+\.\d+\.\d+\\bin\\1cv8\.exe",
        str(file_path),
    )


def test_get_last_1c_exe_file_path_2() -> None:
    with pytest.raises(Exception) as e:
        get_last_1c_exe_file_fullpath(config_file="bla.cfg")

        assert re.match(r"1CEStart.cfg file does not exist", str(e))



# -*- coding: utf-8 -*-
from pathlib import Path

from appdirs import site_data_dir
from cjk_commons.settings import get_path_attribute
from loguru import logger

from commons_1c.version import get_version_as_number

logger.disable(__name__)


def get_last_exe_file_fullpath(file_name: str, **kwargs) -> Path:
    """Получить путь к exe-файлу из последней установленной платформы 1С

    Args:
        - file_name (str) -- Имя файла

    Raises:
        - FileExistsError -- Возникает, если файл не существует

    Returns:
        - Path -- Путь к exe-файлу
    """

    result = None

    config_file_path = get_path_attribute(
        kwargs,
        "config_file_path",
        default_path=Path(site_data_dir("1CEStart", "1C"), "1CEStart.cfg"),
        is_dir=False,
        check_if_exists=False,
    )

    if config_file_path.is_file():
        installed_location_paths = []

        with config_file_path.open(encoding="utf-16") as config_file:
            for line in config_file.readlines():
                key_and_value = line.split("=")
                if key_and_value[0] == "InstalledLocation":
                    value = "=".join(key_and_value[1:])
                    installed_location_paths.append(Path(value.rstrip("\n")))

        platform_versions = []

        for installed_location_path in installed_location_paths:
            if installed_location_path.is_dir():
                # todo
                for version_dir_path in installed_location_path.rglob("*"):
                    version_as_number = get_version_as_number(version_dir_path.name)
                    if version_as_number:
                        exe_file_path = Path(version_dir_path, "bin", file_name)
                        if exe_file_path.is_file():
                            platform_versions.append((version_as_number, exe_file_path))

        platform_versions_reversed = sorted(
            platform_versions, key=lambda x: x[0], reverse=True
        )

        if platform_versions_reversed:
            result = platform_versions_reversed[0][1]
    else:
        raise FileExistsError("1CEStart.cfg file does not exist")

    return result


def get_last_1c_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath("1cv8.exe", **kwargs)


def get_last_ibcmd_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath("ibcmd.exe", **kwargs)


def get_last_rac_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath("rac.exe", **kwargs)



# -*- coding: utf-8 -*-
import re
from typing import List

from loguru import logger

logger.disable(__name__)

pattern_version = re.compile(
    r"\D*(?P<version>(?:(\d+)|)(?:\.(\d+)|)(?:\.(\d+)|)(?:\.(\d+)|))\D*"
)


def get_version_as_number(version: str, m: int = 10000) -> int:
    result = 0

    match = pattern_version.match(version)
    if match is not None:
        a = match.group(2)
        a = "0" if a is None else a

        b = match.group(3)
        b = "0" if b is None else b

        c = match.group(4)
        c = "0" if c is None else c

        d = match.group(5)
        d = "0" if d is None else d

        result = int(a) * m**3 + int(b) * m**2 + int(c) * m + int(d)

    return result


def get_version_as_parts(version: str) -> List[str]:
    result = []
    match = pattern_version.match(version)

    if match is not None:
        a = match.group(2)

        if a is not None:
            result.append(a)

        b = match.group(3)

        if b is not None:
            result.append(b)

        c = match.group(4)

        if c is not None:
            result.append(c)

        d = match.group(5)

        if d is not None:
            result.append(d)

    return result
